string Description = "Phong Lighting";

float4 DiffuseColor : Diffuse
<
    string UIName = "Diffuse Color";
> = {0.6, 0.9, 0.6, 1.0};

float SpecularPower
<
    string UIWidget = "Slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName = "Specular Power";
> = 30.0;

// connect this to point light object
float4 LightPosition : Position
<
    string Object = "PointLight";
    string Space = "World";
> = {-1.0, 2.0, 1.0, 1.0};

float4x4 W   : World;
float4x4 WVP : WorldViewProjection;
float4x4 WIT : WorldInverseTranspose;
float4x4 VI  : ViewInverse;

struct AppData
{
    float3 position : POSITION;
    float3 normal : NORMAL;
};

struct VertexOutput
{
    float4 hPosition : POSITION;
    float4 color : COLOR0;
};

struct PixelOutput
{
    float4 color : COLOR;
};

VertexOutput VS_Phong(AppData IN) 
{
    VertexOutput OUT;

    float4 PObject = float4(IN.position, 1.0);
    
    OUT.hPosition = mul(PObject, WVP);
    
    float3 PWorld = mul(PObject, W).xyz;
    float3 LightVector = normalize(LightPosition.xyz - PWorld);
    float3 NWorld = normalize(mul(float4(IN.normal, 0.0), WIT).xyz);
    float DiffuseIntensity = max(0.0, dot(LightVector, NWorld));
    float3 ViewVector = normalize(VI[3].xyz - PWorld);
    float3 HalfVector = normalize(ViewVector + LightVector);

    OUT.color = DiffuseIntensity * DiffuseColor
                + pow(max(0.0, dot(HalfVector, NWorld)), SpecularPower);

    //float4 lighting = lit(dot(LightVector, NWorld),
    //                      dot(HalfVector, NWorld), SpecularPower);
    //OUT.color = DiffuseColor*lighting.y + lighting.z;

    return OUT;
}

PixelOutput PS_Color(VertexOutput IN)
{
    PixelOutput OUT;
    OUT.color = IN.color;
    return OUT;
}

technique simpleTechnique
{
    pass p0 
    {        
        VertexShader = compile vs_2_0 VS_Phong();
        PixelShader  = compile ps_2_0 PS_Color();
    }
}